@@ -1,29 +1,29 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 import requests
 from bag.console import bool_input
-from nine import input
-from . import ReleaseStep, StopRelease, system, system_or_stop
+from nine import input, nine
+from . import ReleaseStep, StopRelease, CommandStep
 from .regex import version_in_python_source_file
 
 
+@nine
 class Shell(ReleaseStep):
     '''Runs some shell command.'''
     ERROR_CODE = 2
 
     def __init__(self, command, stop_on_failure=True):
-        super(Shell, self).__init__()
-        self.command = command
+        self.COMMAND = command
         self.stop_on_failure = stop_on_failure
+        self.no_rollback = 'Unable to roll back the step {0}'.format(self)
 
     def __call__(self):
-        if self.stop_on_failure:
-            system_or_stop(self.command)
-        else:
-            system(self.command)
+        self._execute_or_complain(self.COMMAND)  # sets self.success
+
+    def __str__(self):
+        return '[' + self.COMMAND + ']'
 
 
 class InteractivelyApproveDistribution(ReleaseStep):
@@ -33,7 +33,7 @@ class InteractivelyApproveDistribution(ReleaseStep):
     ERROR_CODE = 3
 
     def __call__(self):
-        system_or_stop('python setup.py sdist')
+        self._execute_or_complain('python setup.py sdist')
         # TODO: Optionally xdg-open the archive for convenience
         # Create the sdist with "-d <output_dir>"  on a temp dir.
         # Delete it at the end.
@@ -41,7 +41,8 @@ class InteractivelyApproveDistribution(ReleaseStep):
         print("to open the archive (in the 'dist' directory) ")
         print("and check that all files are in there.")
         if not bool_input("Do you approve the archive contents?"):
-            raise StopRelease('If the distribution is missing some files,\n'
+            raise StopRelease('Source distribution content not approved.\n'
+                'If the distribution is missing some files,\n'
                 "try correcting your MANIFEST.in file according to\n"
                 "http://docs.python.org/3/distutils/sourcedist.html"
                 "#specifying-the-files-to-distribute")
@@ -66,8 +67,9 @@ class CheckTravis(ReleaseStep):
                 'Travis has not built branch "{0}" yet.'.format(branch))
         latest = finished[0]
         if latest['result'] == 0:
-            print('No problem in latest Travis build: "{0}"'.format(
-                latest.get('message')))
+            self.log.info('No problem in latest Travis build: "{0}"'.format(
+                          latest.get('message')))
+            self._succeed()
         else:
             raise StopRelease(
                 'Last Travis build on branch "{0}" failed.'.format(branch))
@@ -87,12 +89,34 @@ class SetVersionNumberInteractively(ReleaseStep):
         releaser.the_version = input('What is the new version number? ')
         # Write the new version onto the source code
         version_in_python_source_file(path, replace=releaser.the_version)
+        self._succeed()
+
+
+class PypiRegister(CommandStep):
+    COMMAND = 'python setup.py register'
+    ERROR_CODE = 5
+    no_rollback = 'The release has been created on http://pypi.python.org,\n' \
+        'Unfortunately, the release must be removed manually.'
+
+    def _validate_command_output(self, command_output):
+        return "Server response (200): OK" in command_output
+
+
+class PypiUpload(CommandStep):
+    COMMAND = 'python setup.py sdist upload'
+    ERROR_CODE = 6
+    no_rollback = 'Cannot roll back the sdist upload to http://pypi.python.org'
+
+    def _validate_command_output(self, command_output):
+        return "Server response (200): OK" in command_output
 
 
 class SetFutureVersion(ReleaseStep):
     '''Replaces the version number in the configured source code file with
     the future version number (the one ending in 'dev').
     '''
+    ERROR_CODE = 7
+
     def __call__(self):
         releaser = self.releaser
         path = releaser.config['version_file']
@@ -100,6 +124,13 @@ class SetFutureVersion(ReleaseStep):
         # we can still execute the current step, by populating the_version:
         if releaser.the_version is None:
             releaser._the_version = version_in_python_source_file(path)
-        print('Ready for the next development cycle! Setting version ' +
-              releaser.future_version)
+        self.log.info('Ready for the next development cycle! Setting version '
+              + releaser.future_version)
         version_in_python_source_file(path, replace=releaser.future_version)
+        self._succeed()
+
+
+class ErrorStep(CommandStep):
+    '''Raises an exception to force a rollback. Good for testing.'''
+    COMMAND = 'thisCommandDontExist'
+    ERROR_CODE = 255
